﻿using System.Linq;
using Buddy.BehaviorTree;
using Buddy.CommonBot;
using Buddy.Swtor;
using Buddy.Swtor.Objects;
using WingIt.Dynamics;

namespace WingIt.Routines
{
    public static class OperativeMedic
    {
        #region Constants
        private const double RescueResourceLevel = 45;
        private const double DefaultSelfHealingLevel = 75;
        private const double MinHoTCompanionHealingLevel = 80;
        private const double DefaultCompanionIsDeadMeHealingLevel = 50;
        private const double CriticalMeHealingLevel = 20;
        private const double CriticalCompanionHealingLevel = 20;
        private const double CriticalResourceLevel = 25;
        #endregion

        #region Statics

        private static bool MeHasTacticalAdvantage { get { return BuddyTor.Me.Buffs.FirstOrDefault(b => b.Name == "Tactical Advantage") != null; } }

        #endregion

        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeMedic)]
        public static Composite OperativeMedicPull()
        {
            return new PrioritySelector(
                Movement.StopInRange(Global.RangeDistance),
                Movement.MoveToLineOfSight(),
                Spell.WaitForCast(),

                Spell.Cast("Rifle Shot")
            );
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeMedic)]
        public static Composite OperativeMedicCombat()
        {
            var koltoProbe = BuddyTor.Me.Buffs.FirstOrDefault(b => b.Name == "Kolto Probe");
            return new PrioritySelector(ctx => BuddyTor.Me,

                new Decorator(ret => BuddyTor.Me.CurrentTarget == null && BuddyTor.Me.EnemiesAttackers.Any(), new Action(delegate
                {
                    Logger.Write("No target! ");
                    return RunStatus.Failure;

                })),

                // CD's
                Spell.Cast("Shield Probe", ret => BuddyTor.Me, ret => BuddyTor.Me.HealthPercent <= DefaultSelfHealingLevel), // Absorbs DMG for 15s
                Spell.Cast("Evasion", ret => BuddyTor.Me, ret => BuddyTor.Me.HealthPercent <= CriticalMeHealingLevel), // Immune to DMG for 3s
                Spell.Cast("Stim Boost", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage), // Restores Energy over time
                Spell.Cast("Adrenaline Probe", ret => BuddyTor.Me, ret => BuddyTor.Me.ResourceStat <= RescueResourceLevel), // Restores 50 Energy
                Spell.Cast("Distraction", ret => BuddyTor.Me.CurrentTarget.Range <= 1f && Global.RequiresCrowdControl), // Interrupt
                Spell.Cast("Flash Bang", ret => BuddyTor.Me.CurrentTarget.Range <= 1f && Global.RequiresCrowdControl), // Blind/Interrupt
                Spell.Cast("Debilitate", ret => BuddyTor.Me.CurrentTarget.Range <= Global.MeleeDistance && Global.RequiresCrowdControl), // Stun

                // Do we need to heal ourselves?
                new Decorator(ret => BuddyTor.Me.HealthPercent <= DefaultSelfHealingLevel,
                    new PrioritySelector(
                        ctx => BuddyTor.Me, // Set us as context please. Kthx!

                        Spell.Cast("Surgical Probe", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage && BuddyTor.Me.HealthPercent <= CriticalMeHealingLevel),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Injection", ret => BuddyTor.Me, ret => AbilityManager.CanCast("Kolto Injection", BuddyTor.Me)),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Probe", ret => BuddyTor.Me, ret => !BuddyTor.Me.HasBuff("Kolto Probe") || (koltoProbe != null && koltoProbe.Stacks < 2)), // Fuck null ref exceptions.
                        Spell.Cast("Diagnostic Scan", ret => BuddyTor.Me, ret => BuddyTor.Me.ResourceStat < CriticalResourceLevel)
                        )),

                // Do we need to heal our companion?
                new Decorator(ret => BuddyTor.Me.Companion != null && !BuddyTor.Me.Companion.IsDead && BuddyTor.Me.Companion.HealthPercent <= Global.CompHealing,
                    new PrioritySelector(
                        Spell.Cast("Surgical Probe", ret => BuddyTor.Me.Companion, ret => MeHasTacticalAdvantage && BuddyTor.Me.Companion.HealthPercent <= CriticalCompanionHealingLevel),
                        Spell.Cast("Diagnostic Scan", ret => BuddyTor.Me.Companion, ret => BuddyTor.Me.Companion.HealthPercent >= MinHoTCompanionHealingLevel && BuddyTor.Me.Companion.HealthPercent < Global.CompHealing),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me.Companion, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Injection", ret => BuddyTor.Me.Companion, ret => AbilityManager.CanCast("Kolto Injection", BuddyTor.Me.Companion)),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me.Companion, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Probe", ret => BuddyTor.Me.Companion, ret => !BuddyTor.Me.Companion.HasBuff("Kolto Probe") || (koltoProbe != null && koltoProbe.Stacks < 2)),
                        Spell.Cast("Diagnostic Scan", ret => BuddyTor.Me.Companion, ret => BuddyTor.Me.ResourceStat < CriticalResourceLevel)
                        )),

                        Spell.Cast("Corrosive Dart", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Poisoned (Tech)") && Global.RequiresCrowdControl),

                        Spell.Cast("Fragmentation Grenade", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3),
                        Spell.Cast("Carbine Burst", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3 && BuddyTor.Me.CurrentTarget.Range <= 1f && MeHasTacticalAdvantage),

                        Spell.Cast("Eviscerate"),
                        Spell.Cast("Backstab"),
                        Spell.Cast("Shiv"),

                        Spell.Cast("Crouch", ret => !Global.IsInCover),

                        Spell.Cast("Explosive Probe"),
                        Spell.Cast("Snipe"),
                        Spell.Cast("Overload Shot", ret => !Global.IsInCover),
                        Spell.Cast("Rifle Shot"),

                Movement.MoveTo(ctx => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeMedic)]
        public static Composite OperativeMedicPostCombat()
        {
            // PostCombat handling prevents the 'stuck handler dance' if combat ends in crouch/cover
            return (CommonSubBT.CreateBTPS_AgentOrSmugglerBreakCrouchOrCover());
        }
    }
}