﻿using System;
using System.Linq;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Swtor;
using WingIt.Dynamics;
using Action = Buddy.BehaviorTree.Action;

namespace WingIt.Routines
{
    //Neo93
    public static class SageBalance
    {
        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageBalance)]
        public static Composite SageBalancePull()
        {
            return SageBalanceCombat();
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageBalance)]
        public static Composite SageBalanceCombat()
        {
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.RangeDistance),

                //General
                Spell.Cast("Force of Will", castWhen => BuddyTor.Me.IsStunned),
                Spell.Cast("Force Potency", castWhen => Helpers.Tank.HealthPercent <= Global.HighHealth),//Buff for 2 spells

                //CC
                Spell.Cast("Force Stun", ret => Global.RequiresCrowdControl),
                Spell.Cast("Force Wave", ret => Helpers.MobsAround() >= 3 && Helpers.Targets.Count() >= 3),

                //Healing
                Spell.Cast("Force Armor", on => Helpers.Tank, castWhen => !Global.IsInGroup && !Helpers.Tank.HasBuff("Force Armor") && !Helpers.Tank.HasDebuff("Force-imbalance")),//Shield
                Spell.Cast("Force Armor", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && !Helpers.LowestHealthPlayer.HasBuff("Force Armor") && !Helpers.LowestHealthPlayer.HasDebuff("Force-imbalance") && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Shield
                Spell.Cast("Rejuvenate", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Rejuvenate")),//Low Heal & HoT
                Spell.Cast("Healing Trance", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Low Heal 

                //Offensive
                Spell.CastOnGround("Force in Balance", ret => BuddyTor.Me.CurrentTarget.Distance <= 0.8f, location => BuddyTor.Me.CurrentTarget.Position),
                Spell.Cast("Weaken Mind", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Weaken Mind (Force)")),
                Spell.Cast("Sever Force", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Sever Force")),
                Spell.Cast("Mind Crush", ret => BuddyTor.Me.HasBuff("Presence of Mind")),
                Spell.Cast("Telekinetic Throw"),
                Spell.Cast("Project"),
                Spell.Cast("Disturbance", ret => (AbilityManager.HasAbility("Presence of Mind") && BuddyTor.Me.HasBuff("Presence of Mind")) || !AbilityManager.HasAbility("Presence of Mind")),

                //Movement
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageBalance)]
        public static Composite SageBalanceOutOfCombat()
        {
            return new PrioritySelector(
                    Spell.Cast("Force Speed", ret => BuddyTor.Me.IsMoving),
                    new Decorator(ret => Helpers.LowestHealthPlayer != null && Helpers.LowestHealthPlayer.HealthPercent <= 95,
                        new PrioritySelector(
                            Spell.Cast("Rejuvenate", on => Helpers.LowestHealthPlayer, j => Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Rejuvenate")),//Low Heal & HoT
                            Spell.Cast("Healing Trance", on => Helpers.LowestHealthPlayer, k => Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth)//Low Heal 
                )));
        }

    }
}