﻿using System.Linq;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Swtor;
using Buddy.Swtor.Objects;
using WingIt.Dynamics;

namespace WingIt.Routines
{
    //Neo93
    public static class SorcererLightning
    {
        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererLightning)]
        public static Composite SorcererLightningPull()
        {
            return SorcererLightningCombat();
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererLightning)]
        public static Composite SorcererLightningCombat()
        {
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.RangeDistance),

                //General
                Spell.Cast("Unbreakable Will", castWhen => BuddyTor.Me.IsStunned),
                Spell.Cast("Recklessness"),//Buff for 2 spells

                //CC
                Spell.Cast("Electrocute", ret => Global.RequiresCrowdControl),
                Spell.Cast("Overload", ret => Helpers.MobsAround() >= 3 && Helpers.Targets.Count() >= 3),

                //Healing
                Spell.Cast("Static Barrier", on => Helpers.Tank, ret => !Global.IsInGroup && !Helpers.Tank.HasBuff("Static Barrier") && !Helpers.Tank.HasDebuff("Deionized")),//Shield
                Spell.Cast("Static Barrier", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && !Helpers.LowestHealthPlayer.HasBuff("Static Barrier") && !Helpers.LowestHealthPlayer.HasDebuff("Deionized") && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Shield
                Spell.Cast("Resurgence", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Resurgence")),//Low Heal & HoT
                Spell.Cast("Innervate", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Low Heal & HoT

                //Offensive
                Spell.Cast("Affliction", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Affliction (Force)")),
                Spell.Cast("Lightning Strike", ret => (AbilityManager.HasAbility("Subversion") && BuddyTor.Me.HasBuff("Subversion") && BuddyTor.Me.Buffs.FirstOrDefault(b => b.Name == "Subversion").Stacks < 3) || (AbilityManager.HasAbility("Subversion") && !BuddyTor.Me.HasBuff("Subversion"))),
                Spell.Cast("Crushing Darkness"),
                Spell.Cast("Thundering Blast"),
                Spell.Cast("Chain Lightning", on => BuddyTor.Me.CurrentTarget, ret => Helpers.MobsAround(0.8f) >= 3 || BuddyTor.Me.HasBuff("Lightning Storm")),
                Spell.Cast("Force Lightning", ret => (AbilityManager.HasAbility("Lightning Barrage") && BuddyTor.Me.HasBuff("Lightning Barrage")) || !AbilityManager.HasAbility("Lightning Barrage")),
                Spell.Cast("Shock"),
                Spell.Cast("Lightning Strike"),

                //Movement
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererLightning)]
        public static Composite SorcererLightningOutOfCombat()
        {
            return new PrioritySelector(
                    Spell.Cast("Force Speed", ret => BuddyTor.Me.IsMoving),
                    new Decorator(ret => Helpers.LowestHealthPlayer != null && Helpers.LowestHealthPlayer.HealthPercent <= 95,
                        new PrioritySelector(
                            Spell.Cast("Resurgence", on => Helpers.LowestHealthPlayer, j => Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Resurgence")),//Low Heal & HoT
                            Spell.Cast("Innervate", on => Helpers.LowestHealthPlayer, k => Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth)//Low Heal 
                )));
        }

    }
}