﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Buddy.Swtor;

namespace WingIt
{
    class Tunables
    {
        public delegate bool       PredicateDelegate();

        public class ClassTunables
        {
            public string               RejuvenateName              = "UNDEFINED";
            public string               SelfBuffName                = "UNDEFINED";
            public PredicateDelegate    IsRejuvenationNeeded        = () => { return (false); };
            public PredicateDelegate    IsRejuvenationComplete      = () => { return (true); };
        };


        private static readonly ClassTunables   CompanionTunables = new ClassTunables()
        {
            IsRejuvenationNeeded = () => { return (Helpers.IsCompanionInUse() && !Helpers.Companion.IsDead && (Helpers.Companion.HealthPercent < 70)); },
            IsRejuvenationComplete = () => { return (!Helpers.IsCompanionInUse() || Helpers.Companion.IsDead || (Helpers.Companion.HealthPercent >= 95)); }
        };

        // This unifies handling of class generic buffing & healing...
        // Previously, various (disparate) constants were used inconsistently thoughout the code.
        private static readonly Dictionary<CharacterClass, ClassTunables>  ToonTunables = new Dictionary<CharacterClass, ClassTunables>()
        {
            // Republic
            { CharacterClass.Knight, 
                new ClassTunables() {
                    RejuvenateName = "Introspection",
                    SelfBuffName = "Force Might",
                    IsRejuvenationNeeded = () => { return (BuddyTor.Me.HealthPercent < 70); },
                    IsRejuvenationComplete = () => { return (BuddyTor.Me.HealthPercent >= 95); }
                }},

            { CharacterClass.Consular,
                new ClassTunables() {
                    RejuvenateName = "Meditation",
                    SelfBuffName = "Force Valor",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); }
                }},

            { CharacterClass.Smuggler,
                new ClassTunables() {
                    RejuvenateName = "Recuperate",
                    SelfBuffName = "Lucky Shots",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); }
                }},

            { CharacterClass.Trooper,
                new ClassTunables() {
                    RejuvenateName = "Recharge and Reload",
                    SelfBuffName = "Fortification",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 5)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat > 9)); }
                }},

            // Empire
            { CharacterClass.Warrior,
                new ClassTunables() {
                    RejuvenateName = "Channel Hatred",
                    SelfBuffName = "Unnatural Might",
                    IsRejuvenationNeeded = () => { return (BuddyTor.Me.HealthPercent < 70); },
                    IsRejuvenationComplete = () => { return (BuddyTor.Me.HealthPercent >= 95); }
                }},

            { CharacterClass.Inquisitor,
                new ClassTunables() {
                    RejuvenateName = "Seethe",
                    SelfBuffName = "Mark of Power",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); }
                }},

            { CharacterClass.Agent,
                new ClassTunables() {
                    RejuvenateName = "Recuperate",
                    SelfBuffName = "Coordination",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); }
                }},

            { CharacterClass.BountyHunter,
                new ClassTunables() {
                    RejuvenateName="Recharge and Reload",
                    SelfBuffName="Hunter's Boon",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat > 30)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat <= 5)); }
                }}
        };


        public static ClassTunables     ForCompanion
        {
            get { return (CompanionTunables); }
        }

        public static ClassTunables     ForToon
        {
            get { return (ToonTunables[BuddyTor.Me.Class]); }
        }
    }
}
